@@ -1,5 +1,4 @@
 ###############################################################################
-#
 #  Copyright (C) 2002-2007  Travis Shirk <travis@pobox.com>
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -18,7 +17,6 @@
 #
 ################################################################################
 import re, os, string, stat, shutil, tempfile, binascii;
-import mimetypes;
 from stat import *;
 from eyeD3 import *;
 import eyeD3.utils;
@@ -26,6 +24,7 @@ import eyeD3.mp3;
 from frames import *;
 from binfuncs import *;
 import math;
+from xml.sax.saxutils import escape
 
 ID3_V1_COMMENT_DESC = "ID3 v1 Comment";
 
@@ -131,7 +130,7 @@ class TagHeader:
                                                  self.experimental,
                                                  self.footer));
 
-      # The size of the optional extended header, frames, and padding
+      # The size of the extended header (optional), frames, and padding
       # afer unsynchronization.  This is a sync safe integer, so only the
       # bottom 7 bits of each byte are used.
       tagSizeStr = f.read(4);
@@ -158,7 +157,7 @@ class TagHeader:
       TRACE_MSG("Setting tag size to %d" % tagLen);
       szBytes = bin2bytes(bin2synchsafe(dec2bin(tagLen, 32)));
       data += szBytes;
-      TRACE_MSG("TagHeader Rendered");
+      TRACE_MSG("TagHeader rendered %d bytes" % len(data));
       return data;
 
 ################################################################################
@@ -257,7 +256,7 @@ class ExtendedTagHeader:
        return bites;
 
 
-   def render(self, header, frameData, padding = 0):
+   def render(self, header, frameData, padding=0):
       assert(header.majorVersion == 2);
 
       data = "";
@@ -293,13 +292,13 @@ class ExtendedTagHeader:
          TRACE_MSG("Rendered extended header of size %d" % len(data));
       else:
          # Version 2.3
-         size = 10;
+         size = 6;  # Note, the 4 size bytes are not included in the size
          # Extended flags.
          f = [0] * 16;
          if self.hasCRC():
             f[0] = 1;
             # XXX: Using the absolute value of the CRC.  The spec is unclear
-            # about the type of this type.
+            # about the type of this value.
             self.crc = int(math.fabs(binascii.crc32(frameData +\
                                                     ("\x00" * padding))));
             crc = bin2bytes(dec2bin(self.crc));
@@ -328,8 +327,8 @@ class ExtendedTagHeader:
       if header.minorVersion == 4:
          # sync-safe
          sz = bin2dec(bytes2bin(data, 7));
-         TRACE_MSG("Extended header size (not including 4 byte size): %d" %\
-                   (sz - 4));
+         self.size = sz
+         TRACE_MSG("Extended header size (includes the 4 size bytes): %d" % sz);
          data = fp.read(sz - 4);
 
          if ord(data[0]) != 1 or (ord(data[1]) & 0x8f):
@@ -363,12 +362,12 @@ class ExtendedTagHeader:
       else:
          # v2.3 is totally different... *sigh*
          sz = bin2dec(bytes2bin(data));
-         TRACE_MSG("Extended header size (not including 4 bytes size): %d" %\
-                   sz);
+         TRACE_MSG("Extended header size (not including 4 size bytes): %d" % sz)
+         self.size = sz + 4  # +4 to include size bytes
          tmpFlags = fp.read(2);
          # Read the padding size, but it'll be computed during the parse.
          ps = fp.read(4);
-         TRACE_MSG("Extended header says there is %d bytes of padding" %\
+         TRACE_MSG("Extended header says there is %d bytes of padding" %
                    bin2dec(bytes2bin(ps)));
          # Make this look like a v2.4 mask.
          self.flags = ord(tmpFlags[0]) >> 2;
@@ -403,6 +402,9 @@ class Tag:
    # If this value is None the tag is not linked to any particular file..
    linkedFile = None;
 
+   # add TDTG (or TXXX) - Tagging Time - when saved
+   do_tdtg = True
+
    # Constructor.  An empty tag is created and the link method is used
    # to read an mp3 file's v1.x or v2.x tag.  You can optionally set a 
    # file name, but it will not be read, but may be written to.
@@ -554,8 +556,6 @@ class Tag:
           self.header.tagSize:
          tagFile = file(self.linkedFile.name, "r+b");
          if tagFile.read(3) == "ID3":
-            # FIXME:
-            print "POINT #2"
             TRACE_MSG("Removing ID3 v2.x Tag");
             tagSize = self.header.tagSize + self.header.SIZE;
             tagFile.seek(tagSize);
@@ -670,7 +670,7 @@ class Tag:
       return self._getNum(TRACKNUM_FID)
 
    # Like TrackNum, except for DiscNum--that is, position in a set. Most
-   # songs won't have this or it will be 1/1.
+   # tags won't have this or it will be 1/1.
    def getDiscNum(self):
       return self._getNum(DISCNUM_FID)
 
@@ -698,11 +698,11 @@ class Tag:
    # Both URLFrame and UserURLFrame objects are returned.  UserURLFrames
    # add a description and encoding, and have a different frame ID.
    def getURLs(self):
-      urls = list();
-      for fid in URL_FIDS:
-         urls.extend(self.frames[fid]);
-      urls.extend(self.frames[USERURL_FID]);
-      return urls;
+       urls = []
+       for fid in URL_FIDS:
+           urls.extend(self.frames[fid])
+       urls.extend(self.frames[USERURL_FID])
+       return urls
 
    def getUserTextFrames(self):
       return self.frames[USERTEXT_FID];
@@ -742,10 +742,12 @@ class Tag:
       if not year and not fid:
           dateFrames = self.getDate();
           if dateFrames:
-              self.frames.removeFramesByID(dateFrames[0].header.id);
-          return;
+              self.frames.removeFramesByID(dateFrames[0].header.id)
+          return
       elif not year:
-          self.frames.removeFramesByID(fid);
+          self.frames.removeFramesByID(fid)
+      else:
+          self.frames.removeFramesByID(frames.OBSOLETE_YEAR_FID)
 
       dateStr = self.strToUnicode(str(year));
       if len(dateStr) != 4:
@@ -804,36 +806,36 @@ class Tag:
    # Accepts a tuple with the first value containing the track number and the
    # second the total number of tracks.  One or both of these values may be
    # None.  If both values are None, the frame is removed.
-   def setTrackNum(self, n):
-      self.setNum(TRACKNUM_FID, n)
+   def setTrackNum(self, n, zeropad = True):
+      self.setNum(TRACKNUM_FID, n, zeropad)
 
-   def setDiscNum(self, n):
-      self.setNum(DISCNUM_FID, n)
+   def setDiscNum(self, n, zeropad = True):
+      self.setNum(DISCNUM_FID, n, zeropad)
 
-   def setNum(self, fid, n):
+   def setNum(self, fid, n, zeropad = True):
       if n[0] == None and n[1] == None:
          self.frames.removeFramesByID(fid);
          return;
 
       totalStr = "";
-      zPadding = 1;
       if n[1] != None:
-         if n[1] >= 0 and n[1] <= 9:
+         if zeropad and n[1] >= 0 and n[1] <= 9:
             totalStr = "0" + str(n[1]);
          else:
             totalStr = str(n[1]);
-         zPadding = len(totalStr) - 1;
 
       t = n[0];
       if t == None:
          t = 0;
 
-      # Pad with zeros according to how large the total count is.
       trackStr = str(t);
-      if len(trackStr) == 1:
-         trackStr = "0" + trackStr;
-      if len(trackStr) < len(totalStr):
-         trackStr = ("0" * (len(totalStr) - len(trackStr))) + trackStr;
+
+      # Pad with zeros according to how large the total count is.
+      if zeropad:
+         if len(trackStr) == 1:
+            trackStr = "0" + trackStr;
+         if len(trackStr) < len(totalStr):
+            trackStr = ("0" * (len(totalStr) - len(trackStr))) + trackStr;
 
       s = "";
       if trackStr and totalStr:
@@ -880,21 +882,41 @@ class Tag:
    # Semantics similar to addComment
    def addUserTextFrame(self, desc, text):
       if not text:
-         u_frames = self.frames[USERTEXT_FID];
+         u_frames = self.frames[USERTEXT_FID]
          for u in u_frames:
             if u.description == desc:
                self.frames.remove(u);
-               break;
+               break
       else:
          self.frames.setUserTextFrame(self.strToUnicode(text),
                                       self.strToUnicode(desc), self.encoding);
 
+   def addUserURLFrame(self, desc, url):
+      if not url:
+         u_frames = self.frames[USERURL_FID]
+         for u in u_frames:
+            if u.description == desc:
+               self.frames.remove(u)
+               break
+      else:
+         self.frames.setUserURLFrame(str(url), self.strToUnicode(desc),
+                                     self.encoding)
+
+   def removeUserTextFrame(self, desc):
+      self.addUserTextFrame(desc, "")
+
+   def removeUserURLFrame(self, desc):
+      self.addUserURLFrame(desc, "")
+
    def removeComments(self):
        return self.frames.removeFramesByID(COMMENT_FID);
 
    def removeLyrics(self):
        return self.frames.removeFramesByID(LYRICS_FID);
 
+   def removeImages(self):
+       return self.frames.removeFramesByID(IMAGE_FID)
+
    def addImage(self, type, image_file_path, desc = u""):
        if image_file_path:
            image_frame = ImageFrame.create(type, image_file_path, desc);
@@ -960,12 +982,19 @@ class Tag:
    def getBPM(self):
       bpm = self.frames[BPM_FID];
       if bpm:
-          return int(bpm[0].text);
+          try:
+              bpm = float(bpm[0].text)
+          except ValueError:
+              # Invalid bpm value, in the spirit of not crashing...
+              bpm = 0.0
+          finally:
+              # Round floats since the spec says this is an integer
+              return int(round(bpm))
       else:
           return None;
 
    def setBPM(self, bpm):
-       self.setTextFrame(BPM_FID, self.strToUnicode(str(bpm)));
+       self.setTextFrame(BPM_FID, self.strToUnicode(str(bpm)))
 
    def getPublisher(self):
       pub = self.frames[PUBLISHER_FID];
@@ -997,19 +1026,25 @@ class Tag:
        else:
           self.frames.setTextFrame(fid, self.strToUnicode(txt), self.encoding);
 
+   def setURLFrame(self, fid, url):
+       if not url:
+          self.frames.removeFramesByID(fid)
+       else:
+          self.frames.setURLFrame(fid, url)
+
    def setTextEncoding(self, enc):
        if enc != LATIN1_ENCODING and enc != UTF_16_ENCODING and\
           enc != UTF_16BE_ENCODING and enc != UTF_8_ENCODING:
-           raise TagException("Invalid encoding");
+           raise TagException("Invalid encoding")
        elif self.getVersion() & ID3_V1 and enc != LATIN1_ENCODING:
-           raise TagException("ID3 v1.x supports ISO-8859 encoding only");
+           raise TagException("ID3 v1.x supports ISO-8859 encoding only")
        elif self.getVersion() <= ID3_V2_3 and enc == UTF_8_ENCODING:
            # This is unfortunate.
-           raise TagException("UTF-8 is not supported by ID3 v2.3");
+           raise TagException("UTF-8 is not supported by ID3 v2.3")
 
-       self.encoding = enc;
+       self.encoding = enc
        for f in self.frames:
-           f.encoding = enc;
+           f.encoding = enc
 
    def tagToString(self, pattern):
        # %A - artist
@@ -1017,23 +1052,27 @@ class Tag:
        # %t - title
        # %n - track number
        # %N - track total
-       s = self._subst(pattern, "%A", self.getArtist());
-       s = self._subst(s, "%a", self.getAlbum());
-       s = self._subst(s, "%t", self.getTitle());
-       s = self._subst(s, "%n", self._prettyTrack(self.getTrackNum()[0]));
-       s = self._subst(s, "%N", self._prettyTrack(self.getTrackNum()[1]));
-       return s;
+       # %Y - year 
+       # %G - genre
+       s = self._subst(pattern, "%A", self.getArtist())
+       s = self._subst(s, "%a", self.getAlbum())
+       s = self._subst(s, "%t", self.getTitle())
+       s = self._subst(s, "%n", self._prettyTrack(self.getTrackNum()[0]))
+       s = self._subst(s, "%N", self._prettyTrack(self.getTrackNum()[1]))
+       s = self._subst(s, "%Y", self.getYear())
+       s = self._subst(s, "%G", self.getGenre().name)
+       return s
 
    def _prettyTrack(self, track):
        if not track:
-           return None;
-       track_str = str(track);
+           return None
+       track_str = str(track)
        if len(track_str) == 1:
-           track_str = "0" + track_str;
-       return track_str;
+           track_str = "0" + track_str
+       return track_str
 
    def _subst(self, name, pattern, repl):
-       regex = re.compile(pattern);
+       regex = re.compile(pattern)
        if regex.search(name) and repl:
            # No '/' characters allowed
            (repl, subs) = re.compile("/").subn("-", repl);
@@ -1062,7 +1101,7 @@ class Tag:
          elif c.description == "":
             cmt = c.comment;
             # Keep searching in case we find the description eyeD3 uses.
-      cmt = self._fixToWidth(cmt, 30);
+      cmt = self._fixToWidth(cmt.encode("latin_1"), 30);
       if version != ID3_V1_0:
          track = self.getTrackNum()[0];
          if track != None:
@@ -1179,33 +1218,31 @@ class Tag:
 
       currPadding = 0;
       currTagSize = 0
-      if currTagSize == 0:
-         # We may be converting from 1.x to 2.x so we need to find any
-         # current v2.x tag otherwise we're gonna hork the file.
-         tmpTag = Tag();
-         if tmpTag.link(self.linkedFile.name, ID3_V2):
-            TRACE_MSG("Found current v2.x tag:");
-            currTagSize = tmpTag.linkedFile.tagSize;
-            TRACE_MSG("Current tag size: %d" % currTagSize);
-            currPadding = tmpTag.linkedFile.tagPadding;
-            TRACE_MSG("Current tag padding: %d" % currPadding);
+      # We may be converting from 1.x to 2.x so we need to find any
+      # current v2.x tag otherwise we're gonna hork the file.
+      tmpTag = Tag();
+      if tmpTag.link(self.linkedFile.name, ID3_V2):
+         TRACE_MSG("Found current v2.x tag:");
+         currTagSize = tmpTag.linkedFile.tagSize;
+         TRACE_MSG("Current tag size: %d" % currTagSize);
+         currPadding = tmpTag.linkedFile.tagPadding;
+         TRACE_MSG("Current tag padding: %d" % currPadding);
 
-      # Tag it!
-
-      # Set the tagging time
-      t = time.strftime("%Y-%m-%dT%H:%M:%S", time.gmtime());
-      # TDTG for 2.4
-      if self.header.minorVersion == 4:
-          h = FrameHeader(self.header);
-          h.id = "TDTG";
-          dateFrame = DateFrame(h, date_str = self.strToUnicode(t),
-                                encoding = self.encoding);
-          self.frames.removeFramesByID("TDTG");
-          self.frames.addFrame(dateFrame);
-      # TXXX (Tagging time) for older versions
-      else:
-          self.frames.removeFramesByID("TDTG");
-          self.addUserTextFrame('Tagging time', t)
+      if self.do_tdtg:
+          t = time.strftime("%Y-%m-%dT%H:%M:%S", time.gmtime());
+          # Tag it!
+          if self.header.minorVersion == 4:
+              # TDTG for 2.4
+              h = FrameHeader(self.header);
+              h.id = "TDTG";
+              dateFrame = DateFrame(h, date_str = self.strToUnicode(t),
+                                    encoding = self.encoding);
+              self.frames.removeFramesByID("TDTG");
+              self.frames.addFrame(dateFrame);
+          else:
+              # TXXX (Tagging time) for older versions
+              self.frames.removeFramesByID("TDTG");
+              self.addUserTextFrame('Tagging time', t)
 
       # Render all frames first so the data size is known for the tag header.
       frameData = "";
@@ -1220,38 +1257,46 @@ class Tag:
           TRACE_MSG("Unsyncing all frames (sync-safe)");
           frameData = frames.unsyncData(frameData);
 
-      TRACE_MSG("Rendered tag size: " + str(len(frameData)));
-
       rewriteFile = 0;
       paddingSize = 0;
-      headerData = "";
-      extHeaderData = "";
+      DEFAULT_PADDING = 1024
+      def compute_padding():
+          if currPadding <= DEFAULT_PADDING:
+              return DEFAULT_PADDING
+          else:
+              return currPadding
 
       # Extended header
+      extHeaderData = "";
       if self.header.extended:
          # This is sorta lame.  We don't know the total framesize until
          # this is rendered, yet we can't render it witout knowing the
-         # amount of padding.  Force it.
+         # amount of padding for the crc.  Force it.
          rewriteFile = 1;
-         paddingSize = 2048;
          TRACE_MSG("Rendering extended header");
+         paddingSize = compute_padding()
          extHeaderData += self.extendedHeader.render(self.header, frameData,
                                                      paddingSize);
 
-      if rewriteFile or (10 + len(headerData) + len(extHeaderData) +\
-                         len(frameData)) >= currTagSize:
+      new_size = 10 + len(extHeaderData) + len(frameData) + paddingSize
+      if rewriteFile or new_size >= currTagSize:
          TRACE_MSG("File rewrite required");
          rewriteFile = 1;
-         paddingSize = 2048;
-      else:
-         paddingSize = currTagSize - (len(headerData) + len(extHeaderData) +\
-                                      len(frameData));
+         if paddingSize <= 0:
+             paddingSize = compute_padding()
+      elif paddingSize <= 0:
+         paddingSize = currTagSize - (new_size - 10)
+      TRACE_MSG("Adding %d bytes of padding" % paddingSize)
       frameData += ("\x00" * paddingSize);
 
+      # Recompute with padding
+      new_size = 10 + len(extHeaderData) + len(frameData)
+      header_tag_size = new_size - 10
+
       # Render the tag header.
-      TRACE_MSG("Rendering %s tag header with size %d" %\
-                (versionToString(self.getVersion()), len(frameData)));
-      headerData = self.header.render(len(frameData));
+      TRACE_MSG("Rendering %s tag header with size %d" %
+                (versionToString(self.getVersion()), header_tag_size))
+      headerData = self.header.render(header_tag_size)
 
       # Assemble frame.
       tagData = headerData + extHeaderData + frameData;
@@ -1263,8 +1308,6 @@ class Tag:
          tagFile.write(tagData);
          tagFile.close();
       else:
-         # FIXME
-         print "POINT #1"
          # Open tmp file
          tmpName = tempfile.mktemp();
          tmpFile = file(tmpName, "w+b");
@@ -1273,7 +1316,13 @@ class Tag:
 
          # Write audio data in chunks
          tagFile = file(self.linkedFile.name, "rb");
-         tagFile.seek(currTagSize);
+         if currTagSize != 0:
+             seek_point = currTagSize + 10
+         else:
+             seek_point = 0
+         TRACE_MSG("Seeking to beginning of audio data, byte %d (%x)" %
+                   (seek_point, seek_point))
+         tagFile.seek(seek_point)
          self.__copyRemaining(tagFile, tmpFile);
 
          tagFile.close();
@@ -1312,12 +1361,11 @@ class Tag:
 
          # Header is definitely there so at least one frame *must* follow.
          self.frames.setTagHeader(self.header);
-         padding = self.frames.parse(fp, self.header);
+         padding = self.frames.parse(fp, self.header, self.extendedHeader);
          TRACE_MSG("Tag contains %d bytes of padding." % padding);
       except FrameException, ex:
-         if utils.strictID3():
-            fp.close();
-            raise TagException(str(ex));
+         fp.close();
+         raise TagException(str(ex));
       except TagException:
          fp.close();
          raise;
@@ -1420,8 +1468,9 @@ class Genre:
 
       try:
          id = genres[name];
-         # Get titled case.
-         name = genres[id];
+         if (not utils.itunesCompat()) or (id <= GenreMap.ITUNES_GENRE_MAX):
+            # Get titled case.
+            name = genres[id];
       except:
           if utils.strictID3():
               raise GenreException("Invalid genre name: " + name);
@@ -1503,17 +1552,34 @@ class Genre:
          return;
 
       # Non standard, but witnessed.
-      # Match genreName alone.  e.g. Rap, Rock, blues.
-      regex = re.compile("^[A-Z 0-9'+/\-&]+\00*$", re.IGNORECASE);
+      # Match genre alone. e.g. Rap, Rock, blues, 'Rock|Punk|Pop-Punk', etc
+      '''
+      regex = re.compile("^[A-Z 0-9+/\-\|!&'\.]+\00*$", re.IGNORECASE)
       if regex.match(genreStr):
+         print "boo"
          self.setName(genreStr);
          return;
+     '''
+
+      # non standard, but witnessed. 
+      # Match delimited-separated list of genres alone.
+      # e.g. 'breaks, electronic', 'hardcore|nyhc' 
+      regex = re.compile(
+              r"^([A-Z 0-9+/\-\|!&'\.]+)([,;|][A-Z 0-9+/\-\|!&'\.]+)*$",
+              re.IGNORECASE)
+      if regex.match(genreStr):
+         print "boo"
+         self.setName(genreStr);
+         return;
+
       raise GenreException("Genre string cannot be parsed with '%s': %s" %\
                            (regex.pattern, genreStr));
 
    def __str__(self):
       s = "";
-      if self.id != None:
+      if (self.id != None and
+              ((not utils.itunesCompat()) or
+               (self.id <= GenreMap.ITUNES_GENRE_MAX))):
          s += "(" + str(self.id) + ")"
       if self.name:
          s += self.name;
@@ -1549,6 +1615,11 @@ class TagFile:
        if not dir:
            dir = ".";
        new_name = dir + os.sep + name.encode(fsencoding) + base_ext;
+
+       if os.path.exists(new_name):
+           raise TagException("File '%s' exists, eyeD3 will not overwrite it" %
+                              new_name)
+
        try:
            os.rename(self.fileName, new_name);
            self.fileName = new_name;
@@ -1560,120 +1631,89 @@ class TagFile:
       return self.play_time;
 
    def getPlayTimeString(self):
-      total = self.getPlayTime();
-      h = total / 3600;
-      m = (total % 3600) / 60;
-      s = (total % 3600) % 60;
-      if h:
-         timeStr = "%d:%.2d:%.2d" % (h, m, s);
-      else:
-         timeStr = "%d:%.2d" % (m, s);
-      return timeStr;
+      from eyeD3.utils import format_track_time
+      return format_track_time(self.getPlayTime())
 
 
 ################################################################################
 class Mp3AudioFile(TagFile):
-   header         = mp3.Header();
-   xingHeader     = None;
-   invalidFileExc = InvalidAudioFormatException("File is not mp3");
 
    def __init__(self, fileName, tagVersion = ID3_ANY_VERSION):
-      TagFile.__init__(self, fileName);
+      TagFile.__init__(self, fileName)
+
+      self.tag        = None
+      self.header     = None
+      self.xingHeader = None
+      self.lameTag    = None
 
       if not isMp3File(fileName):
-         raise self.invalidFileExc;
+         raise InvalidAudioFormatException("File is not mp3");
 
       # Parse ID3 tag.
       f = file(self.fileName, "rb");
-      tag = Tag();
-      hasTag = tag.link(f, tagVersion);
+      self.tag = Tag();
+      hasTag = self.tag.link(f, tagVersion);
       # Find the first mp3 frame.
-      if tag.isV1():
+      if self.tag.isV1():
          framePos = 0;
       elif not hasTag:
          framePos = 0;
-         tag = None;
+         self.tag = None;
       else:
-         # XXX: Note that v2.4 allows for appended tags; account for that.
-         framePos = tag.header.SIZE + tag.header.tagSize;
+         framePos = self.tag.header.SIZE + self.tag.header.tagSize;
 
-      f.seek(framePos);
-      bString = f.read(4);
-      if len(bString) < 4:
-         raise InvalidAudioFormatException("Unable to find a valid mp3 "\
-                                           "frame");
-      frameHead = bin2dec(bytes2bin(bString));
-      header = mp3.Header();
-      it_count = 0;
-      # Keep reading until we find a valid mp3 frame header.
-      while not header.isValid(frameHead):
-         # Originally, the search was one byte at a time.  Occasionally a tag
-         # was heavily, and incorrectly, padded or a corrupt mp3 would cause
-         # a byte by byte search to take a long time.  This algorithm speeds
-         # up this particular case.
-         if it_count > 9:
-             f.seek(f.tell() + 128);
-             bString = f.read(4);
-             if len(bString) < 4:
-                raise InvalidAudioFormatException("Unable to find a valid mp3 "\
-                                                  "frame");
-             frameHead = bin2dec(bytes2bin(bString));
-             it_count = 0;
-             continue;
+      TRACE_MSG("mp3 header search starting @ %x" % framePos)
+      # Find an mp3 header
+      header_pos, header, header_bytes = mp3.find_header(f, framePos)
+      if header:
+          try:
+              self.header = mp3.Header(header)
+          except mp3.Mp3Exception, ex:
+              self.header = None
+              raise InvalidAudioFormatException(str(ex));
+          else:
+              TRACE_MSG("mp3 header %x found at position: 0x%x" % (header,
+                                                                   header_pos))
+      else:
+        raise InvalidAudioFormatException("Unable to find a valid mp3 frame")
 
-         frameHead <<= 8;
-         bString = f.read(1);
-         if len(bString) != 1:
-            raise InvalidAudioFormatException("Unable to find a valid mp3 "\
-                                              "frame");
-         frameHead |= ord(bString[0]);
-         it_count += 1;
-      TRACE_MSG("mp3 header %x found at position: %d (0x%x)" % \
-                (frameHead, f.tell() - 4, f.tell() - 4));
-
-      # Decode the header.
-      try:
-         header.decode(frameHead);
-         # Check for Xing header inforamtion which will always be in the
-         # first "null" frame.
-         f.seek(-4, 1);
-         mp3Frame = f.read(header.frameLength);
-         if mp3Frame.find("Xing") != -1:
-            xingHeader = mp3.XingHeader();
-            if not xingHeader.decode(mp3Frame):
-               raise InvalidAudioFormatException("Corrupt Xing header");
-         else:
-            xingHeader = None;
-      except mp3.Mp3Exception, ex:
-         raise InvalidAudioFormatException(str(ex));
+      # Check for Xing/Info header information which will always be in the
+      # first "null" frame.
+      f.seek(header_pos)
+      mp3_frame = f.read(self.header.frameLength)
+      if re.compile('Xing|Info').search(mp3_frame):
+          self.xingHeader = mp3.XingHeader();
+          if not self.xingHeader.decode(mp3_frame):
+              TRACE_MSG("Ignoring corrupt Xing header")
+              self.xingHeader = None
+      # Check for LAME Tag
+      self.lameTag = mp3.LameTag(mp3_frame)
 
       # Compute track play time.
-      tpf = mp3.computeTimePerFrame(header);
-      if xingHeader:
-         self.play_time = int(tpf * xingHeader.numFrames);
+      tpf = mp3.computeTimePerFrame(self.header);
+      if self.xingHeader and self.xingHeader.vbr:
+         self.play_time = int(tpf * self.xingHeader.numFrames);
       else:
          length = self.getSize();
-         if tag and tag.isV2():
-            length -= tag.header.SIZE + tag.header.tagSize;
+         if self.tag and self.tag.isV2():
+            length -= self.tag.header.SIZE + self.tag.header.tagSize;
             # Handle the case where there is a v2 tag and a v1 tag.
             f.seek(-128, 2)
             if f.read(3) == "TAG":
                length -= 128;
-         elif tag and tag.isV1():
+         elif self.tag and self.tag.isV1():
             length -= 128;
-         self.play_time = int((length / header.frameLength) * tpf);
+         self.play_time = int((length / self.header.frameLength) * tpf);
 
-      self.header = header;
-      self.xingHeader = xingHeader;
-      self.tag = tag;
       f.close();
 
    # Returns a tuple.  The first value is a boolean which if true means the
    # bit rate returned in the second value is variable.
    def getBitRate(self):
       xHead = self.xingHeader;
-      if xHead:
+      if xHead and xHead.vbr:
          tpf = eyeD3.mp3.computeTimePerFrame(self.header);
+         # FIXME: if xHead.numFrames == 0 (Fuoco.mp3), ZeroDivisionError
          br = int((xHead.numBytes * 8) / (tpf * xHead.numFrames * 1000));
          vbr = 1;
       else:
@@ -1692,7 +1732,7 @@ class Mp3AudioFile(TagFile):
 
 ################################################################################
 def isMp3File(fileName):
-    (type, enc) = mimetypes.guess_type(fileName);
+    type = eyeD3.utils.guess_mime_type(fileName);
     return type == "audio/mpeg";
 
 ################################################################################
@@ -1704,6 +1744,7 @@ class GenreMap(list):
    ID3_GENRE_MAX = 79;
    WINAMP_GENRE_MIN = 80;
    WINAMP_GENRE_MAX = 147;
+   ITUNES_GENRE_MAX = 125;
    EYED3_GENRE_MIN = None;
    EYED3_GENRE_MAX = None;
 
@@ -1917,21 +1958,45 @@ def tagToUserTune(tag):
         audio_file = tag;
         tag = audio_file.getTag();
 
-    tune =  u"<tune xmlns='http://jabber.org/protocol/tune'>\n";
-    if tag.getArtist():
-        tune += "  <artist>" + tag.getArtist() + "</artist>\n";
-    if tag.getTitle():
-        tune += "  <title>" + tag.getTitle() + "</title>\n";
-    if tag.getAlbum():
-        tune += "  <source>" + tag.getAlbum() + "</source>\n";
-    tune += "  <track>" +\
-            "file://" + unicode(os.path.abspath(tag.linkedFile.name)) +\
-            "</track>\n";
+    tune =  [u"<tune xmlns='http://jabber.org/protocol/tune'>\n"]
+    def add(name, value):
+        if value:
+            value = escape(value)
+            tune.append('  <%s>%s</%s>\n' % (name, value, name))
+  
+    add('artist', tag.getArtist())
+    add('title', tag.getTitle())
+    add('source', tag.getAlbum())
+    add('track', "file://" + unicode(os.path.abspath(tag.linkedFile.name)))
     if audio_file:
-        tune += "  <length>" + unicode(audio_file.getPlayTime()) +\
-                "</length>\n";
-    tune += "</tune>\n";
-    return tune;
+        add('length', unicode(audio_file.getPlayTime()))
+    tune.append("</tune>\n")
+    return ''.join(tune)
+
+def tagToRfc822(tag):
+    if isinstance(tag, Mp3AudioFile):
+        tag = tag.getTag();
+    lines = []
+    def add(name, value):
+        if value:
+            lines.append(u'%s: %s\n' % (name, value))
+    add('Filename', tag.linkedFile.name)
+    add('Artist', tag.getArtist())
+    add('Album', tag.getAlbum())
+    for comment in tag.getComments():
+        add('Comment', comment.comment)
+    try:
+        g = tag.getGenre()
+        if g:
+            add('Genre', '%s (%s)' % (g.getName(), g.getId() or 0))
+    except GenreException:
+        pass
+    add('Title', tag.getTitle())
+    tn, tt = tag.getTrackNum()
+    add('Track', tn)
+    add('Year', tag.getYear())
+    return ''.join(lines)
+
 
 #
 # Module level globals.
